package cz.zcu.kiv.db.dao.jdbc;

import cz.zcu.kiv.db.dao.*;
import cz.zcu.kiv.db.dao.jdbc.model.*;
import cz.zcu.kiv.db.model.osoby.*;
import java.util.ArrayList;
import java.util.List;
import javax.sql.DataSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.simple.ParameterizedBeanPropertyRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;
import org.springframework.stereotype.Repository;

/**
 * Implementace rozhrani OsobaDao za pouziti Spring Jdbc.
 * Anotace @Repository("osobaDao") ulozi instanci teto tridy do IOC kontejneru springu,
 * kde bude k dispozici pod identifikatorem "osobaDao".
 * <code>OsobaDao oDao = (OsobaDao) ctx.getBean("osobaDao");</code>
 *  
 * @author Radek Muzika
 */
@Repository("osobaDao")
public class OsobaDaoImpl implements OsobaDao {

    /**
     * SimpleJdbcTemplate umoznuje snadnou manipulaci s databazi. Vice viz.
     * dokumentace springu http://static.springframework.org/spring/docs/2.5.x/reference/jdbc.html#jdbc-SimpleJdbcTemplate
     */
    private SimpleJdbcTemplate simpleJdbcTemplate;
    /**
     * SimpleJdbcInsert umoznuje snadne vkladani dat do databaze a nasledne ziskani
     * automaticky generovaneho primarniho klice (pokud takovy je). Vice viz.
     * dokumentace springu http://static.springframework.org/spring/docs/2.5.x/reference/jdbc.html#jdbc-simple-jdbc-insert-1
     */
    private SimpleJdbcInsert osobaInsert;
    private SimpleJdbcInsert clenstviInsert;
    private SimpleJdbcInsert poziceInsert;
    private SimpleJdbcInsert aktivitaInsert;

    /**
     * Anotace @Autowired zajistuje automaticke instanciovani objektu z IOC kontejneru.
     * Coz znamena ze atributu _temaDao bude v instanci tridy, ulozenem v IOC kontejneru
     * pod identifikatorem "osobaDao" automaticky prirazena instance tridy TemaDao, ktera
     * je jiz ulozena v IOC kontejneru.
     */
    @Autowired
    private TemaDao _temaDao;    
    @Autowired
    private UdalostDao _udalostDao;
    @Autowired
    private VyzkumDao _vyzkumDao;
    @Autowired
    private PublikaceDao _publikaceDao;
    @Autowired
    private StudiumDao _studiumDao;
    
    
    /**
     * Anotace @Autowired pouzita u metody init je dalsim moznym pouzitim IOC. Beana z IOC kontejneru
     * je vlozena do volani metody jako parametr. V tomto pripade se jedna o dataSource,
     * nadefinovany v konfiguracnim souboru.
     * @param dataSource
     */
    @Autowired
    public void init(DataSource dataSource) {
        this.simpleJdbcTemplate = new SimpleJdbcTemplate(dataSource);
        this.osobaInsert = new SimpleJdbcInsert(dataSource).withTableName("personal_osoby").usingGeneratedKeyColumns("id");
        this.clenstviInsert = new SimpleJdbcInsert(dataSource).withTableName("personal_clenstvi").usingGeneratedKeyColumns("id");
        this.poziceInsert = new SimpleJdbcInsert(dataSource).withTableName("personal_pozice").usingGeneratedKeyColumns("id");
        this.aktivitaInsert = new SimpleJdbcInsert(dataSource).withTableName("personal_aktivity").usingGeneratedKeyColumns("id");
    }

    public Osoba getOsoba(int id) {
        String sql = "select * from personal_osoby where id=?";
        try {
            JdbcOsoba osoba = this.simpleJdbcTemplate.queryForObject(
                    sql,
                    ParameterizedBeanPropertyRowMapper.newInstance(JdbcOsoba.class),
                    id);
            /* Objektu osoba musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. */
            osoba.setOsobaDao(this);
            osoba.setTemaDao(_temaDao);
            osoba.setUdalostDao(_udalostDao);
            osoba.setStudiumDao(_studiumDao);
            osoba.setVyzkumDao(_vyzkumDao);
            osoba.setPublikaceDao(_publikaceDao);
            return osoba;
        } catch (org.springframework.dao.EmptyResultDataAccessException ex) {
            return null;
        }
    }

    public Clenstvi getClenstvi(int id) {
        String sql = "select * from personal_clenstvi where id=?";
        try {
            JdbcClenstvi clenstvi = this.simpleJdbcTemplate.queryForObject(
                    sql,
                    ParameterizedBeanPropertyRowMapper.newInstance(JdbcClenstvi.class),
                    id);
            /* Objektu clenstvi musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. */
            clenstvi.setOsobaDao(this);
            return clenstvi;
        } catch (org.springframework.dao.EmptyResultDataAccessException ex) {
            return null;
        }
    }

    public Aktivita getAktivita(int id) {
        String sql = "select * from personal_aktivita where id=?";
        try {
            JdbcAktivita aktivita = this.simpleJdbcTemplate.queryForObject(
                    sql,
                    ParameterizedBeanPropertyRowMapper.newInstance(JdbcAktivita.class),
                    id);
            /* Objektu aktivita musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. */
            aktivita.setOsobaDao(this);
            return aktivita;
        } catch (org.springframework.dao.EmptyResultDataAccessException ex) {
            return null;
        }
    }

    public Pozice getPozice(int id) {
        String sql = "select * from personal_pozice where id=?";
        try {
            JdbcPozice pozice = this.simpleJdbcTemplate.queryForObject(
                    sql,
                    ParameterizedBeanPropertyRowMapper.newInstance(JdbcPozice.class),
                    id);
            /* Objektu pozice musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. */
            pozice.setOsobaDao(this);
            return pozice;
        } catch (org.springframework.dao.EmptyResultDataAccessException ex) {
            return null;
        }
    }

    public List<Osoba> getOsoby() {
        return this.getOsoby(false);
    }

    public List<Osoba> getOsoby(boolean vcetneNeaktivnich) {
        String sql = "select * from personal_osoby" +
                (vcetneNeaktivnich ? "" : " where aktivni=1") +
                " order by prijmeni,jmeno";
        List<JdbcOsoba> jdbclst = this.simpleJdbcTemplate.query(sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcOsoba.class));
        List<Osoba> lst = new ArrayList<Osoba>();
        for (JdbcOsoba osoba : jdbclst) {
            /* Objektu osoba musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. */
            osoba.setOsobaDao(this);
            osoba.setTemaDao(_temaDao);
            osoba.setUdalostDao(_udalostDao);
            osoba.setStudiumDao(_studiumDao);
            osoba.setVyzkumDao(_vyzkumDao);
            osoba.setPublikaceDao(_publikaceDao);
            lst.add(osoba);
        }
        return lst;
    }

    public List<Clenstvi> getClenstvi() {
        return this.getClenstvi(false);
    }

    public List<Clenstvi> getClenstvi(boolean vcetneNeaktivnich) {
        String sql = "select * from personal_clenstvi" +
                (vcetneNeaktivnich ? "" : " where aktivni=1") +
                " order by razeni";
        List<JdbcClenstvi> jdbclst = this.simpleJdbcTemplate.query(sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcClenstvi.class));
        List<Clenstvi> lst = new ArrayList<Clenstvi>();
        for (JdbcClenstvi c : jdbclst) {
            /* Objektu clenstvi musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. */
            c.setOsobaDao(this);
            lst.add(c);
        }
        return lst;
    }

    public List<Aktivita> getAktivity() {
        return this.getAktivity(false);
    }

    public List<Aktivita> getAktivity(boolean vcetneNeaktivnich) {
        String sql = "select * from personal_aktivity" +
                (vcetneNeaktivnich ? "" : " where aktivni=1") +
                " order by razeni";
        List<JdbcAktivita> jdbclst = this.simpleJdbcTemplate.query(sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcAktivita.class));
        List<Aktivita> lst = new ArrayList<Aktivita>();
        for (JdbcAktivita a : jdbclst) {
            /* Objektu aktivita musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. */
            a.setOsobaDao(this);
            lst.add(a);
        }
        return lst;
    }

    public List<Pozice> getPozice() {
        return this.getPozice(false);
    }

    public List<Pozice> getPozice(boolean vcetneNeaktivnich) {
        String sql = "select * from personal_pozice" +
                (vcetneNeaktivnich ? "" : " where aktivni=1") +
                " order by razeni";
        List<JdbcPozice> jdbclst = this.simpleJdbcTemplate.query(sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcPozice.class));
        List<Pozice> lst = new ArrayList<Pozice>();
        for (JdbcPozice p : jdbclst) {
            /* Objektu pozice musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. */
            p.setOsobaDao(this);
            lst.add(p);
        }
        return lst;
    }

    public List<Aktivita> getAktivityDleOsoby(Osoba osoba) {
        return this.getAktivityDleOsoby(osoba, false);
    }

    public List<Aktivita> getAktivityDleOsoby(Osoba osoba, boolean vcetneNeaktivnich) {
        String sql = "select a.* from personal_aktivity_osob as ao " +
                "left join personal_aktivity as a " +
                "on a.id=ao.fk_aktivita " +
                "where ao.fk_osoba=? " +
                (vcetneNeaktivnich ? "" : " and a.aktivni=1 ") +
                "order by a.razeni";
        List<JdbcAktivita> jdbclst = this.simpleJdbcTemplate.query(sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcAktivita.class),
                osoba.getId());
        List<Aktivita> lst = new ArrayList<Aktivita>();
        for (JdbcAktivita a : jdbclst) {
            /* Objektu a musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. */
            a.setOsobaDao(this);
            lst.add(a);
        }
        return lst;
    }

    public List<Clenstvi> getClenstviDleOsoby(Osoba osoba) {
        return this.getClenstviDleOsoby(osoba, false);
    }

    public List<Clenstvi> getClenstviDleOsoby(Osoba osoba, boolean vcetneNeaktivnich) {
        String sql = "select c.* from personal_clenstvi_osob as co " +
                "left join personal_clenstvi as c " +
                "on c.id=co.fk_clenstvi " +
                "where co.fk_osoba=? " +
                (vcetneNeaktivnich ? "" : " and c.aktivni=1 ") +
                "order by c.razeni";
        List<JdbcClenstvi> jdbclst = this.simpleJdbcTemplate.query(sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcClenstvi.class),
                osoba.getId());
        List<Clenstvi> lst = new ArrayList<Clenstvi>();
        for (JdbcClenstvi c : jdbclst) {
            /* Objektu c musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. */
            c.setOsobaDao(this);
            lst.add(c);
        }
        return lst;
    }

    public List<Pozice> getPoziceDleOsoby(Osoba osoba) {
        return this.getPoziceDleOsoby(osoba, false);
    }

    public List<Pozice> getPoziceDleOsoby(Osoba osoba, boolean vcetneNeaktivnich) {
        String sql = "select p.* from personal_funkce as f " +
                "left join personal_pozice as p " +
                "on p.id=f.fk_pozice " +
                "where f.fk_osoba=? " +
                (vcetneNeaktivnich ? "" : " and p.aktivni=1 ") +
                "order by p.razeni";
        List<JdbcPozice> jdbclst = this.simpleJdbcTemplate.query(sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcPozice.class),
                osoba.getId());
        List<Pozice> lst = new ArrayList<Pozice>();
        for (JdbcPozice p : jdbclst) {
            /* Objektu p musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. */
            p.setOsobaDao(this);
            lst.add(p);
        }
        return lst;
    }

    public List<Osoba> getOsobyDleClenstvi(Clenstvi clenstvi) {
        return this.getOsobyDleClenstvi(clenstvi, false);
    }

    public List<Osoba> getOsobyDleClenstvi(Clenstvi clenstvi, boolean vcetneNeaktivnich) {
        String sql = "select o.* from personal_clenstvi_osob as co " +
                "left join personal_osoby as o " +
                "on co.fk_osoba=o.id " +
                "where co.fk_clenstvi=?" +
                (vcetneNeaktivnich ? "" : " and o.aktivni=1") +
                " order by o.prijmeni,o.jmeno";
        List<JdbcOsoba> jdbclst = this.simpleJdbcTemplate.query(sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcOsoba.class),
                clenstvi.getId());
        List<Osoba> lst = new ArrayList<Osoba>();
        for (JdbcOsoba osoba : jdbclst) {
            /* Objektu osoba musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. */
            osoba.setOsobaDao(this);
            osoba.setTemaDao(_temaDao);
            osoba.setUdalostDao(_udalostDao);
            osoba.setStudiumDao(_studiumDao);
            osoba.setVyzkumDao(_vyzkumDao);
            osoba.setPublikaceDao(_publikaceDao);
            lst.add(osoba);
        }
        return lst;
    }

    public List<Osoba> getOsobyDleAktivity(Aktivita aktivita) {
        return this.getOsobyDleAktivity(aktivita, false);
    }

    public List<Osoba> getOsobyDleAktivity(Aktivita aktivita, boolean vcetneNeaktivnich) {
        String sql = "select o.* from personal_aktivity_osob as ao " +
                "left join personal_osoby as o " +
                "on ao.fk_osoba=o.id " +
                "where ao.fk_aktivita=?" +
                (vcetneNeaktivnich ? "" : " and o.aktivni=1") +
                " order by o.prijmeni,o.jmeno";
        List<JdbcOsoba> jdbclst = this.simpleJdbcTemplate.query(sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcOsoba.class),
                aktivita.getId());
        List<Osoba> lst = new ArrayList<Osoba>();
        for (JdbcOsoba osoba : jdbclst) {
            /* Objektu osoba musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. */
            osoba.setOsobaDao(this);
            osoba.setTemaDao(_temaDao);
            osoba.setUdalostDao(_udalostDao);
            osoba.setStudiumDao(_studiumDao);
            osoba.setVyzkumDao(_vyzkumDao);
            osoba.setPublikaceDao(_publikaceDao);
            lst.add(osoba);
        }
        return lst;
    }

    public List<Osoba> getOsobyDlePozice(Pozice pozice) {
        return this.getOsobyDlePozice(pozice, false);
    }

    public List<Osoba> getOsobyDlePozice(Pozice pozice, boolean vcetneNeaktivnich) {
        String sql = "select o.* from personal_funkce as f " +
                "left join personal_osoby as o " +
                "on f.fk_osoba=o.id " +
                "where f.fk_pozice=?" +
                (vcetneNeaktivnich ? "" : " and o.aktivni=1") +
                " order by o.prijmeni,o.jmeno";
        List<JdbcOsoba> jdbclst = this.simpleJdbcTemplate.query(sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcOsoba.class),
                pozice.getId());
        List<Osoba> lst = new ArrayList<Osoba>();
        for (JdbcOsoba osoba : jdbclst) {
            /* Objektu osoba musime priradit vsechny potrebne dao tridy, ktere potrebuje
             * k dodatecnym dotazum do databaze. */
            osoba.setOsobaDao(this);
            osoba.setTemaDao(_temaDao);
            osoba.setUdalostDao(_udalostDao);
            osoba.setStudiumDao(_studiumDao);
            osoba.setVyzkumDao(_vyzkumDao);
            osoba.setPublikaceDao(_publikaceDao);
            lst.add(osoba);
        }
        return lst;
    }

    public void ulozOsobu(Osoba osoba) {
        if (osoba.isNovyZaznam()) {
            osoba.setId(osobaInsert.executeAndReturnKey(
                    this.getOsobaParameterSource(osoba)).intValue());
        } else {
            String sql = "update personal_osoby set " +
                    "login=:login, " +
                    "titul_pred=:titul_pred, " +
                    "jmeno=:jmeno, " +
                    "prijmeni=:prijmeni, " +
                    "titulZa=:titulZa, " +
                    "je_nyni_clenem=:je_nyni_clenem, " +
                    "email=:email, " +
                    "telefon=:telefon, " +
                    "kancelar=:kancelar," +
                    "url=:url, " +
                    "zivotopis_cz=:zivotopis_cz, " +
                    "zivotopis_en=:zivotopis_en, " +
                    "uredni_hodiny=:uredni_hodiny, " +
                    "je_externi=:je_externi, " +
                    "ext_pracoviste_cz=:ext_pracoviste_cz," +
                    "ext_pracoviste_en=:ext_pracoviste_en, " +
                    "fk_skolitel=:fk_skolitel " +
                    "where id=:id";
            this.simpleJdbcTemplate.update(
                    sql,
                    this.getOsobaParameterSource(osoba));
        }
    }

    /**
     * MapSqlParameterSource slouzi k namapovani objektu na jmenne parametry sql dotazu
     * (Napriklad: select * from udalosti where nazev_cz=:nazev_cz)
     * @param osoba
     * @return
     */
    private MapSqlParameterSource getOsobaParameterSource(Osoba osoba) {
        return new MapSqlParameterSource().addValue("id", osoba.getId()).addValue("login", osoba.getLogin()).addValue("titul_pred", osoba.getTitulPred()).addValue("jmeno", osoba.getJmeno()).addValue("prijmeni", osoba.getPrijmeni()).addValue("titulZa", osoba.getTitulZa()).addValue("je_nyni_clenem", osoba.isJeNyniClenem() ? 1 : 0).addValue("email", osoba.getEmail()).addValue("telefon", osoba.getTelefon()).addValue("kancelar", osoba.getKancelar()).addValue("url", osoba.getUrl()).addValue("zivotopis_cz", osoba.getZivotopisCz()).addValue("zivotopis_en", osoba.getZivotopisEn()).addValue("uredni_hodiny", osoba.getUredniHodiny()).addValue("je_externi", osoba.isJeExterni() ? 1 : 0).addValue("ext_pracoviste_cz", osoba.getExtPracovisteCz()).addValue("ext_pracoviste_en", osoba.getExtPracovisteEn()).addValue("fk_skolitel", osoba.getSkolitel() != null ? osoba.getSkolitel().getId() : new Integer(0));
    }

    public void ulozAktivitu(Aktivita aktivita) {
        if (aktivita.isNovyZaznam()) {
            aktivita.setId(aktivitaInsert.executeAndReturnKey(
                    this.getAktivitaParameterSource(aktivita)).intValue());
        } else {
            String sql = "update personal_aktivity set " +
                    "nazev_cz=:nazev_cz, " +
                    "nazev_en=:nazev_en, " +
                    "razeni=:razeni " +
                    "where id=:id";
            this.simpleJdbcTemplate.update(
                    sql,
                    this.getAktivitaParameterSource(aktivita));
        }
    }
    
    /**
     * MapSqlParameterSource slouzi k namapovani objektu na jmenne parametry sql dotazu
     * (Napriklad: select * from udalosti where nazev_cz=:nazev_cz)
     * @param aktivita
     * @return
     */
    private MapSqlParameterSource getAktivitaParameterSource(Aktivita aktivita) {
        return new MapSqlParameterSource().addValue("id", aktivita.getId()).addValue("nazev_cz", aktivita.getNazevCz()).addValue("nazev_en", aktivita.getNazevEn()).addValue("razeni", aktivita.getRazeni());
    }

    public void ulozClenstvi(Clenstvi clenstvi) {
        if (clenstvi.isNovyZaznam()) {
            clenstvi.setId(osobaInsert.executeAndReturnKey(
                    this.getClenstviParameterSource(clenstvi)).intValue());
        } else {
            String sql = "update personal_clenstvi set " +
                    "nazev_cz=:nazev_cz, " +
                    "nazev_en=:nazev_en, " +
                    "razeni=:razeni " +
                    "where id=:id";
            this.simpleJdbcTemplate.update(
                    sql,
                    this.getClenstviParameterSource(clenstvi));
        }
    }

    /**
     * MapSqlParameterSource slouzi k namapovani objektu na jmenne parametry sql dotazu
     * (Napriklad: select * from udalosti where nazev_cz=:nazev_cz)
     * @param clenstvi
     * @return
     */
    private MapSqlParameterSource getClenstviParameterSource(Clenstvi clenstvi) {
        return new MapSqlParameterSource().addValue("id", clenstvi.getId()).addValue("nazev_cz", clenstvi.getNazevCz()).addValue("nazev_en", clenstvi.getNazevEn()).addValue("razeni", clenstvi.getRazeni());
    }

    public void ulozPozici(Pozice pozice) {
        if (pozice.isNovyZaznam()) {
            pozice.setId(osobaInsert.executeAndReturnKey(
                    this.getPoziceParameterSource(pozice)).intValue());
        } else {
            String sql = "update personal_pozice set " +
                    "nazev_cz=:nazev_cz, " +
                    "nazev_en=:nazev_en, " +
                    "popis_cz=:popis_cz, " +
                    "popis_en=:popis_en, " +
                    "razeni=:razeni " +
                    "where id=:id";
            this.simpleJdbcTemplate.update(
                    sql,
                    this.getPoziceParameterSource(pozice));
        }
    }

    /**
     * MapSqlParameterSource slouzi k namapovani objektu na jmenne parametry sql dotazu
     * (Napriklad: select * from udalosti where nazev_cz=:nazev_cz)
     * 
     * @param udalost
     * @return namapovany objekt
     */
    private MapSqlParameterSource getPoziceParameterSource(Pozice pozice) {
        return new MapSqlParameterSource().addValue("id", pozice.getId()).addValue("nazev_cz", pozice.getNazevCz()).addValue("nazev_en", pozice.getNazevEn()).addValue("popis_cz", pozice.getPopisCz()).addValue("popis_en", pozice.getPopisEn()).addValue("razeni", pozice.getRazeni());
    }

    /**
     * Smaze zaznam z databaze.
     * @param osoba
     */
    public void smazOsobu(Osoba osoba) {
        String sql = "delete from personal_osoby where id=?";
        this.simpleJdbcTemplate.update(sql, osoba.getId());
    }

    /**
     * Smaze zaznam z databaze.
     * @param aktivita
     */
    public void smazAktivitu(Aktivita aktivita) {
        String sql = "delete from personal_aktivity where id=?";
        this.simpleJdbcTemplate.update(sql, aktivita.getId());
    }

    /**
     * Smaze zaznam z databaze.
     * @param clenstvi
     */
    public void smazClenstvi(Clenstvi clenstvi) {
        String sql = "delete from personal_clenstvi where id=?";
        this.simpleJdbcTemplate.update(sql, clenstvi.getId());
    }

    /**
     * Smaze zaznam z databaze.
     * @param pozice
     */
    public void smazPozici(Pozice pozice) {
        String sql = "delete from personal_pozice where id=?";
        this.simpleJdbcTemplate.update(sql, pozice.getId());
    }

    /**
     * Prida osobe aktivitu
     * @param aktivita
     * @param osoba
     */
    public void pridejAktivituOsobe(Aktivita aktivita, Osoba osoba) {
        this.simpleJdbcTemplate.update("insert into personal_aktivity_osob(fk_osoba,fk_aktivita) values(?,?)",
                new Object[]{osoba.getId(), aktivita.getId()});
    }

    /**
     * Prida osobe clenstvi
     * @param clenstvi
     * @param osoba
     */
    public void pridejClenstviOsobe(Clenstvi clenstvi, Osoba osoba) {
        this.simpleJdbcTemplate.update("insert into personal_clenstvi_osob(fk_osoba,fk_clenstvi) values(?,?)",
                new Object[]{osoba.getId(), clenstvi.getId()});
    }

    /**
     * Prida osobe pozici
     * @param pozice
     * @param osoba
     */
    public void pridejPoziciOsobe(Pozice pozice, Osoba osoba) {
        this.simpleJdbcTemplate.update("insert into personal_funkce(fk_osoba,fk_pozice) values(?,?)",
                new Object[]{osoba.getId(), pozice.getId()});
    }

    /**
     * Nastavi sloupec "aktivni" u daneho zaznamu na hodnotu 0
     * @param osoba
     */
    public void deaktivujOsobu(Osoba osoba) {
        String sql = "update personal_osoby set aktivni=0 where id=:id";
        this.simpleJdbcTemplate.update(
                sql,
                this.getOsobaParameterSource(osoba));
    }

    /**
     * Nastavi sloupec "aktivni" u daneho zaznamu na hodnotu 0
     * @param clenstvi
     */
    public void deaktivujClenstvi(Clenstvi clenstvi) {
        String sql = "update personal_clenstvi set aktivni=0 where id=:id";
        this.simpleJdbcTemplate.update(
                sql,
                this.getClenstviParameterSource(clenstvi));
    }

    /**
     * Nastavi sloupec "aktivni" u daneho zaznamu na hodnotu 0
     * @param aktivita
     */
    public void deaktivujAktivitu(Aktivita aktivita) {
        String sql = "update personal_aktivity set aktivni=0 where id=:id";
        this.simpleJdbcTemplate.update(
                sql,
                this.getAktivitaParameterSource(aktivita));
    }

    /**
     * Nastavi sloupec "aktivni" u daneho zaznamu na hodnotu 0
     * @param pozice
     */
    public void deaktivujPozici(Pozice pozice) {
        String sql = "update personal_pozice set aktivni=0 where id=:id";
        this.simpleJdbcTemplate.update(
                sql,
                this.getPoziceParameterSource(pozice));
    }

    /**
     * Odebere osobe aktivitu
     * @param aktivita
     * @param osoba
     */
    public void odeberAktivituOsobe(Aktivita aktivita, Osoba osoba) {
        String sql = "delete from personal_aktivity_osob where fk_aktivita=? and fk_osoba=?";
        this.simpleJdbcTemplate.update(sql, new Object[]{aktivita.getId(), osoba.getId()});
    }

    /**
     * Odebere osobe clenstvi
     * @param clenstvi
     * @param osoba
     */
    public void odeberClenstviOsobe(Clenstvi clenstvi, Osoba osoba) {
        String sql = "delete from personal_clenstvi_osob where fk_clenstvi=? and fk_osoba=?";
        this.simpleJdbcTemplate.update(sql, new Object[]{clenstvi.getId(), osoba.getId()});
    }

    /**
     * Odebere osobe pozici
     * @param pozice
     * @param osoba
     */
    public void odeberPoziciOsobe(Pozice pozice, Osoba osoba) {
        String sql = "delete from personal_funkce where fk_pozice=? and fk_osoba=?";
        this.simpleJdbcTemplate.update(sql, new Object[]{pozice.getId(), osoba.getId()});
    }
}
