
package cz.zcu.kiv.db.dao.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.simple.ParameterizedBeanPropertyRowMapper;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;
import org.springframework.stereotype.Repository;

import cz.zcu.kiv.db.dao.OsobaDao;
import cz.zcu.kiv.db.dao.PublikaceDao;
import cz.zcu.kiv.db.dao.StudiumDao;
import cz.zcu.kiv.db.dao.TemaDao;
import cz.zcu.kiv.db.dao.UdalostDao;
import cz.zcu.kiv.db.dao.VyzkumDao;
import cz.zcu.kiv.db.dao.jdbc.model.JdbcObor;
import cz.zcu.kiv.db.dao.jdbc.model.JdbcOsoba;
import cz.zcu.kiv.db.dao.jdbc.model.JdbcPredmet;
import cz.zcu.kiv.db.dao.jdbc.model.JdbcProgram;
import cz.zcu.kiv.db.model.osoby.Osoba;
import cz.zcu.kiv.db.model.studium.Obor;
import cz.zcu.kiv.db.model.studium.Predmet;
import cz.zcu.kiv.db.model.studium.PredmetVOboru;
import cz.zcu.kiv.db.model.studium.Program;
import cz.zcu.kiv.db.model.studium.VyukaPredmetu;

/**
 * Implementace rozhrani StudiumDao za pouziti Spring Jdbc.
 * Anotace @Repository("studiumDao") ulozi instanci teto tridy do IOC kontejneru springu,
 * kde bude k dispozici pod identifikatorem "studiumDao".
 * <code>StudiumDao sDao = (StudiumDao) ctx.getBean("studiumDao");</code>
 *  
 * @author Radek Muzika
 */
@Repository("studiumDao")
public class StudiumDaoImpl implements StudiumDao{

    /**
     * SimpleJdbcTemplate umoznuje snadnou manipulaci s databazi. Vice viz.
     * dokumentace springu http://static.springframework.org/spring/docs/2.5.x/reference/jdbc.html#jdbc-SimpleJdbcTemplate
     */
    private SimpleJdbcTemplate simpleJdbcTemplate;
    /**
     * SimpleJdbcInsert umoznuje snadne vkladani dat do databaze a nasledne ziskani
     * automaticky generovaneho primarniho klice (pokud takovy je). Vice viz.
     * dokumentace springu http://static.springframework.org/spring/docs/2.5.x/reference/jdbc.html#jdbc-simple-jdbc-insert-1
     */
    private SimpleJdbcInsert predmetInsert;
    private SimpleJdbcInsert oborInsert;
    private SimpleJdbcInsert programInsert;

    /**
     * Anotace @Autowired zajistuje automaticke instanciovani objektu z IOC kontejneru.
     * Coz znamena ze atributu _osobaDao bude v instanci tridy, ulozenem v IOC kontejneru
     * pod identifikatorem "studiumDao" automaticky prirazena instance tridy OsobaDao, ktera
     * je jiz ulozena v IOC kontejneru.
     */
    @Autowired
    private OsobaDao _osobaDao;
    @Autowired
    private TemaDao _temaDao; 
    @Autowired
    private UdalostDao _udalostDao;
    @Autowired
    private VyzkumDao _vyzkumDao;
    @Autowired
    private PublikaceDao _publikaceDao;
    
    /**
     * Anotace @Autowired pouzita u metody init je dalsim moznym pouzitim IOC. Beana z IOC kontejneru
     * je vlozena do volani metody jako parametr. V tomto pripade se jedna o dataSource,
     * nadefinovany v konfiguracnim souboru.
     * @param dataSource
     */
    @Autowired
    public void init(DataSource dataSource) {
        this.simpleJdbcTemplate = new SimpleJdbcTemplate(dataSource);
        this.programInsert = new SimpleJdbcInsert(dataSource)
                .withTableName("studijni_programy")
                .usingGeneratedKeyColumns("id");
        this.oborInsert = new SimpleJdbcInsert(dataSource)
                .withTableName("studijni_obory")
                .usingGeneratedKeyColumns("id");
        this.predmetInsert = new SimpleJdbcInsert(dataSource)
                .withTableName("studijni_predmety")
                .usingGeneratedKeyColumns("id");
    }
    
    
    public List<Obor> getObory() {
        return this.getObory(false);
    }

    public List<Obor> getObory(boolean vcetneNeaktivnich) {
        String sql = "select * from studijni_obory "+
                (vcetneNeaktivnich?"":" where aktivni=1")+
                " order by zkratka";
        List<JdbcObor> jdbclst = this.simpleJdbcTemplate.query(
                sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcObor.class));
        List<Obor> lst = new ArrayList<Obor>();
        for(JdbcObor obj: jdbclst) {
            obj.setStudiumDao(this);
            obj.setTemaDao(_temaDao);
            lst.add(obj);
        }
        return lst;
    }

    public List<Predmet> getPredmety() {
        return this.getPredmety(false);
    }

    public List<Predmet> getPredmety(boolean vcetneNeaktivnich) {
        String sql = "select * from studijni_predmety"+
                (vcetneNeaktivnich?"":" where aktivni=1")+
                " order by zkratka";
        List<JdbcPredmet> jdbclst = this.simpleJdbcTemplate.query(
                sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcPredmet.class));
        List<Predmet> lst = new ArrayList<Predmet>();
        for(JdbcPredmet obj: jdbclst) {
            obj.setStudiumDao(this);
            lst.add(obj);
        }
        return lst;
    }

    public List<Program> getProgramy() {
        return this.getProgramy(false);
    }

    public List<Program> getProgramy(boolean vcetneNeaktivnich) {
        String sql = "select * from studijni_programy"+
                (vcetneNeaktivnich?"":" where aktivni=1")+
                " order by zkratka";
        List<JdbcProgram> jdbclst = this.simpleJdbcTemplate.query(
                sql,
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcProgram.class));
        List<Program> lst = new ArrayList<Program>();
        for(JdbcProgram obj: jdbclst) {
            obj.setStudiumDao(this);
            lst.add(obj);
        }
        return lst;
    }

    public Obor getObor(int id) {
        String sql = "select * from studijni_obory where id = ?";
        try {
            JdbcObor obor = this.simpleJdbcTemplate.queryForObject(
                    sql,
                    ParameterizedBeanPropertyRowMapper.newInstance(JdbcObor.class),
                    id);
            obor.setStudiumDao(this);
            obor.setTemaDao(_temaDao);
            return obor;
        } catch (org.springframework.dao.EmptyResultDataAccessException ex) {
            return null;
        }
    }

    public Program getProgram(int id) {
        String sql = "select * from studijni_programy where id = ?";
        try {
            JdbcProgram program = this.simpleJdbcTemplate.queryForObject(
                    sql,
                    ParameterizedBeanPropertyRowMapper.newInstance(JdbcProgram.class),
                    id);
            program.setStudiumDao(this);
            return program;
        } catch (org.springframework.dao.EmptyResultDataAccessException ex) {
            return null;
        }
    }

    public Predmet getPredmet(int id) {
        String sql = "select * from studijni_predmety where id = ?";
        try {
            JdbcPredmet predmet = this.simpleJdbcTemplate.queryForObject(
                    sql,
                    ParameterizedBeanPropertyRowMapper.newInstance(JdbcPredmet.class),
                    id);
            predmet.setStudiumDao(this);
            return predmet;
        } catch (org.springframework.dao.EmptyResultDataAccessException ex) {
            return null;
        }
    }

    public List<Obor> getOboryDleProgramu(Program program) {
        return this.getOboryDleProgramu(program, false);
    }

    public List<Obor> getOboryDleProgramu(Program program, boolean vcetneNeaktivnich) {
        String sql = "select * from studijni_obory " +
                "where fk_program=? "+
                (vcetneNeaktivnich?"":" and obor.aktivni=1 ")+
                "order by obor.zkratka";
        List<JdbcObor> jdbclst = this.simpleJdbcTemplate.query(
                sql, 
                ParameterizedBeanPropertyRowMapper.newInstance(JdbcObor.class),
                program.getId());
        List<Obor> lst = new ArrayList<Obor>();
        for(JdbcObor obor: jdbclst) {
            obor.setStudiumDao(this);
            obor.setTemaDao(_temaDao);
            lst.add(obor);
        }
        return lst;
    }

    public void ulozObor(Obor obor) {
        if (obor.isNovyZaznam()) {
            obor.setId(oborInsert.executeAndReturnKey(
                    this.getOborParameterSource(obor)).intValue());
        } else {
            String sql = "update studijni_obory set " +
                    "nazev_cz=:nazev_cz, " +
                    "nazev_en=:nazev_en, " +
                    "zkratka=:zkratka, " +
                    "fk_program=:fk_program " +                    
                    "where id=:id";
            this.simpleJdbcTemplate.update(
                    sql,
                    this.getOborParameterSource(obor));
        }
    }

    private MapSqlParameterSource getOborParameterSource(Obor obor) {
        return new MapSqlParameterSource()
                .addValue("id", obor.getId())
                .addValue("nazev_cz", obor.getNazevCz())
                .addValue("nazev_en", obor.getNazevEn())
                .addValue("zkratka", obor.getZkratka())
                .addValue("fk_osoba", obor.getProgram() != null ? 
                    obor.getProgram().getId() : new Integer(0));
    }
    
    public void ulozPredmet(Predmet predmet) {
        if (predmet.isNovyZaznam()) {
            predmet.setId(oborInsert.executeAndReturnKey(
                    this.getPredmetParameterSource(predmet)).intValue());
        } else {
            String sql = "update studijni_predmety set " +
                    "nazev=:nazev, " +
                    "url=:url, " +
                    "zkratka=:zkratka, " +
                    "rok=:rok, " +
                    "semestr=:semestr, " +
                    "je_vyucovan=:je_vyucovan " +                    
                    "where id=:id";
            this.simpleJdbcTemplate.update(
                    sql,
                    this.getPredmetParameterSource(predmet));
        }
    }
    
    private MapSqlParameterSource getPredmetParameterSource(Predmet predmet) {
        return new MapSqlParameterSource()
                .addValue("id", predmet.getId())
                .addValue("nazev_cz", predmet.getNazevCz())
                .addValue("url", predmet.getUrl())
                .addValue("zkratka", predmet.getZkratka())
                .addValue("rok", predmet.getRok())
                .addValue("semestr", predmet.getSemestr())
                .addValue("je_vyucovan", predmet.isJeVyucovan()?1:0);
    }

    public void ulozProgram(Program program) {
        if (program.isNovyZaznam()) {
            program.setId(oborInsert.executeAndReturnKey(
                    this.getProgramParameterSource(program)).intValue());
        } else {
            String sql = "update studijni_programy set " +
                    "nazev_cz=:nazev_cz, " +
                    "nazev_en=:nazev_en, " +
                    "zkratka=:zkratka, " +
                    "je_prezencni=:je_prezencni " +                    
                    "where id=:id";
            this.simpleJdbcTemplate.update(
                    sql,
                    this.getProgramParameterSource(program));
        }
    }
    
    private MapSqlParameterSource getProgramParameterSource(Program program) {
        return new MapSqlParameterSource()
                .addValue("id", program.getId())
                .addValue("nazev_cz", program.getNazevCz())
                .addValue("nazev_en", program.getNazevEn())
                .addValue("zkratka", program.getZkratka())
                .addValue("je_prezencni", program.isJePrezencni()?1:0);
    }

    /**
     * Smaze zaznam z databaze.
     * @param obor
     */
    public void smazObor(Obor obor) {
        String sql = "delete from studijni_obory where id=?";
        this.simpleJdbcTemplate.update(sql, obor.getId());
    }

    /**
     * Smaze zaznam z databaze.
     * @param predmet
     */
    public void smazPredmet(Predmet predmet) {
        String sql = "delete from studijni_predmety where id=?";
        this.simpleJdbcTemplate.update(sql, predmet.getId());
    }

    /**
     * Smaze zaznam z databaze.
     * @param program
     */
    public void smazProgram(Program program) {
        String sql = "delete from studijni_programy where id=?";
        this.simpleJdbcTemplate.update(sql, program.getId());
    }

    /**
     * Nastavi sloupec "aktivni" u daneho zaznamu na hodnotu 0
     * @param obor
     */
    public void deaktivujObor(Obor obor) {
        String sql = "update studijni_obory set aktivni=0 where id=:id";
        this.simpleJdbcTemplate.update(sql, obor.getId());
    }
    
    /**
     * Nastavi sloupec "aktivni" u daneho zaznamu na hodnotu 0
     * @param program
     */
    public void deaktivujProgram(Program program) {
        String sql = "update studijni_programy set aktivni=0 where id=:id";
        this.simpleJdbcTemplate.update(sql, program.getId());
    }

    /**
     * Nastavi sloupec "aktivni" u daneho zaznamu na hodnotu 0
     * @param predmet
     */
    public void deaktivujPredmet(Predmet predmet) {
        String sql = "update studijni_predmety set aktivni=0 where id=:id";
        this.simpleJdbcTemplate.update(sql, predmet.getId());
    }

    public List<VyukaPredmetu> getVyukaPredmetuDleOsoby(Osoba osoba) {
        return this.getVyukaPredmetuDleOsoby(osoba, false);
    }

    public List<VyukaPredmetu> getVyukaPredmetuDleOsoby(Osoba osoba, boolean vcetneNeaktivnich) {
        String sql = "select predmet.*, vyuka.je_prednasejicim from studijni_vyucuje as vyuka " +
                "left join studijni_predmety as predmet " +
                "on vyuka.fk_predmet=predmet.id " +
                "where vyuka.fk_osoba=? "+
                (vcetneNeaktivnich?"":" and predmet.aktivni=1 ")+
                "order by predmet.zkratka";
        List<VyukaPredmetu> jdbclst = this.simpleJdbcTemplate.query(
                sql,
                new ParameterizedRowMapper() {
                    public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
                        VyukaPredmetu vyukaPredmetu = new VyukaPredmetu();
                        ParameterizedBeanPropertyRowMapper pb = ParameterizedBeanPropertyRowMapper.newInstance(JdbcPredmet.class);
                        vyukaPredmetu.setPredmet((JdbcPredmet) pb.mapRow(rs, rowNum));
                        vyukaPredmetu.setVyucujiciJePrednasejici(rs.getInt("je_prednasejicim")==1);
                        return vyukaPredmetu;
                    }
                },
                osoba.getId());
        List<VyukaPredmetu> lst = new ArrayList<VyukaPredmetu>();
        for(VyukaPredmetu vyukaPredmetu: jdbclst) {
            vyukaPredmetu.setVyucujici(osoba);
            ((JdbcPredmet)vyukaPredmetu.getPredmet()).setStudiumDao(this);            
            lst.add(vyukaPredmetu);
        }
        return lst;
    }

    public List<VyukaPredmetu> getVyukaPredmetuDlePredmetu(Predmet predmet) {
        return this.getVyukaPredmetuDlePredmetu(predmet, false);
    }

    public List<VyukaPredmetu> getVyukaPredmetuDlePredmetu(Predmet predmet, boolean vcetneNeaktivnich) {
        String sql = "select osoba.*, vyuka.je_prednasejicim from studijni_vyucuje as vyuka " +
                "left join studijni_osoby as osoba " +
                "on vyuka.fk_osoba=osoba.id " +
                "where vyuka.fk_predmet=? "+
                (vcetneNeaktivnich?"":" and osoba.aktivni=1 ")+
                "order by osoba.prijmeni, osoba.jmeno";
        List<VyukaPredmetu> jdbclst = this.simpleJdbcTemplate.query(
                sql,
                new ParameterizedRowMapper() {
                    public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
                        VyukaPredmetu vyukaPredmetu = new VyukaPredmetu();
                        ParameterizedBeanPropertyRowMapper pb = ParameterizedBeanPropertyRowMapper.newInstance(JdbcOsoba.class);
                        vyukaPredmetu.setVyucujici((JdbcOsoba) pb.mapRow(rs, rowNum));
                        vyukaPredmetu.setVyucujiciJePrednasejici(rs.getInt("je_prednasejicim")==1);
                        return vyukaPredmetu;
                    }
                },
                predmet.getId());
        List<VyukaPredmetu> lst = new ArrayList<VyukaPredmetu>();
        for(VyukaPredmetu vyukaPredmetu: jdbclst) {
            vyukaPredmetu.setPredmet(predmet);
            ((JdbcOsoba)vyukaPredmetu.getVyucujici()).setStudiumDao(this);
            ((JdbcOsoba)vyukaPredmetu.getVyucujici()).setOsobaDao(_osobaDao);
            ((JdbcOsoba)vyukaPredmetu.getVyucujici()).setUdalostDao(_udalostDao);
            ((JdbcOsoba)vyukaPredmetu.getVyucujici()).setVyzkumDao(_vyzkumDao);
            ((JdbcOsoba)vyukaPredmetu.getVyucujici()).setPublikaceDao(_publikaceDao);
            ((JdbcOsoba)vyukaPredmetu.getVyucujici()).setTemaDao(_temaDao);
            lst.add(vyukaPredmetu);
        }
        return lst;
    }

    public List<PredmetVOboru> getPredmetVOboruDleOboru(Obor obor) {
        return this.getPredmetVOboruDleOboru(obor, false);
    }

    public List<PredmetVOboru> getPredmetVOboruDleOboru(Obor obor, boolean vcetneNeaktivnich) {
        String sql = "select predmet.*, po.status from studijni_predmet_obor as po " +
                "left join studijni_predmety as predmet " +
                "on po.fk_predmet=predmet.id " +
                "where po.fk_obor=? "+
                (vcetneNeaktivnich?"":" and predmet.aktivni=1 ")+
                "order by predmet.zkratka";
        List<PredmetVOboru> jdbclst = this.simpleJdbcTemplate.query(
                sql,
                new ParameterizedRowMapper() {
                    public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
                        PredmetVOboru predmetVOboru = new PredmetVOboru();
                        ParameterizedBeanPropertyRowMapper pb = ParameterizedBeanPropertyRowMapper.newInstance(JdbcPredmet.class);
                        JdbcPredmet predmet = (JdbcPredmet) pb.mapRow(rs, rowNum);
                        predmetVOboru.setPredmet(predmet);
                        predmetVOboru.setStatus(rs.getString("status"));
                        return predmetVOboru;
                    }
                },
                obor.getId());
        List<PredmetVOboru> lst = new ArrayList<PredmetVOboru>();
        for(PredmetVOboru predmetVOboru: jdbclst) {
            predmetVOboru.setObor(obor);
            ((JdbcPredmet)predmetVOboru.getPredmet()).setStudiumDao(this);
            lst.add(predmetVOboru);
        }
        return lst;
    }

    public List<PredmetVOboru> getPredmetVOboruDlePredmetu(Predmet predmet) {
        return this.getPredmetVOboruDlePredmetu(predmet, false);
    }

    public List<PredmetVOboru> getPredmetVOboruDlePredmetu(Predmet predmet, boolean vcetneNeaktivnich) {
        String sql = "select obor.*, po.status from studijni_predmet_obor as po " +
                "left join studijni_obory as obor " +
                "on po.fk_obor=obor.id " +
                "where po.fk_predmetu=? "+
                (vcetneNeaktivnich?"":" and obor.aktivni=1 ")+
                "order by obor.zkratka";
        List<PredmetVOboru> jdbclst = this.simpleJdbcTemplate.query(
                sql,
                new ParameterizedRowMapper() {
                    public Object mapRow(ResultSet rs, int rowNum) throws SQLException {
                        PredmetVOboru predmetVOboru = new PredmetVOboru();
                        ParameterizedBeanPropertyRowMapper pb = ParameterizedBeanPropertyRowMapper.newInstance(JdbcObor.class);
                        predmetVOboru.setObor((JdbcObor) pb.mapRow(rs, rowNum));
                        predmetVOboru.setStatus(rs.getString("status"));
                        return predmetVOboru;
                    }
                },
                predmet.getId());
        List<PredmetVOboru> lst = new ArrayList<PredmetVOboru>();
        for(PredmetVOboru predmetVOboru: jdbclst) {
            predmetVOboru.setPredmet(predmet);
            ((JdbcObor)predmetVOboru.getObor()).setStudiumDao(this);
            ((JdbcObor)predmetVOboru.getObor()).setTemaDao(_temaDao);
            lst.add(predmetVOboru); 
        }
        return lst;
    }

    public void pridejVyucujicihoPredmetu(VyukaPredmetu vyuka) {
        this.simpleJdbcTemplate.update("insert into studijni_vyucuje(fk_osoba,fk_predmet,je_prednasejicim) " +
                "values(?,?,?)",
                new Object[]{vyuka.getVyucujici().getId(), 
                        vyuka.getPredmet().getId(), 
                        vyuka.isVyucujiciJePrednasejici()?1:0});
    }

    public void odeberVyucujicihoPredmetu(VyukaPredmetu vyuka) {
        String sql = "delete from studijni_vyucuje where fk_osoba=? and fk_predmet=?";
        this.simpleJdbcTemplate.update(sql, 
                new Object[]{vyuka.getVyucujici().getId(), 
                        vyuka.getPredmet().getId()});
    }

    public void pridejPredmetDoOboru(PredmetVOboru predmetVOboru) {
        this.simpleJdbcTemplate.update("insert into studijni_predmet_obor(fk_predmet,fk_obor,status) " +
                "values(?,?,?)",
                new Object[]{predmetVOboru.getPredmet().getId(), 
                        predmetVOboru.getObor().getId(), 
                        predmetVOboru.getStatus()});
    }

    public void odeberPredmetZOboru(PredmetVOboru predmetVOboru) {
        String sql = "delete from studijni_predmet_obor where fk_predmet=? and fk_obor=?";
        this.simpleJdbcTemplate.update(sql, 
                new Object[]{predmetVOboru.getPredmet().getId(), 
                        predmetVOboru.getObor().getId()});
    }



}
